@@ -33,8 +33,6 @@ PyAPI_FUNC(PyCodeObject *) PyAST_Compile(struct _mod *, const char *,
 					PyCompilerFlags *, PyArena *);
 PyAPI_FUNC(PyFutureFeatures *) PyFuture_FromAST(struct _mod *, const char *);
 
-#define ERR_LATE_FUTURE \
-"from __future__ imports must occur at the beginning of the file"
 
 #ifdef __cplusplus
 }
